from typing import Generator


def slice_list(list_to_chunk: list | tuple, chunk_size: int) -> Generator:
    """
    Slice an iterable (List/Tuple) returning tuples of 'chunk_sixe' size.

    :param list_to_chunk: iterable to slice
    :param chunk_size: Size of the chunk
    :return: Generator of 'chunk_size' sizes of the list.
    """
    n = max(1, chunk_size)
    return (list_to_chunk[i:i+n] for i in range(0, len(list_to_chunk), n))


import os
import sys

from openpyxl.styles.builtins import output

from biobroker.authenticator import WebinAuthenticator # Biosamples uses the WebinAuthenticator
from biobroker.api import BsdApi # BioSamples Database (BSD) API
from biobroker.metadata_entity import Biosample # The metadata entity
from biobroker.input_processor import TsvInputProcessor # An input processor
from biobroker.output_processor import XlsxOutputProcessor # An output processor

def create_sample_tsv():
    sample_tsv = [
        ["sample_id", "collected_at", "organism"],
        ["sumple", "noon", "Homo sapiens"]
    ]
    writable_sample = "\n".join(["\t".join(row) for row in sample_tsv])
    with open("simple_sample_sumple.tsv", "w") as f:
        f.write(writable_sample)

def load_samples_tsv():
    input_processor = TsvInputProcessor("simple_sample_sumple.tsv")
    return input_processor

def update_samples(submitted_samples: list[Biosample], api: BsdApi):
    for sample in submitted_samples:
        sample['newField'] = "newValue"
    return api.update(submitted_samples)

def main(webin_username, webin_password):
    create_sample_tsv()
    # Step 1: load your samples
    input_processor =  load_samples_tsv()
    ## 1.1: transform your metadata
    my_field_map = {"sample_id": "name"}
    input_processor.transform(field_mapping=my_field_map)
    ## 1.2: Process them into Biosample objects
    my_samples = input_processor.process(Biosample)

    # Step 2: Set-up authenticator + webin
    os.environ['API_ENVIRONMENT'] = "dev"  # Dev
    authenticator = WebinAuthenticator(username=webin_username, password=webin_password)
    api = BsdApi(authenticator=authenticator)

    # Step 3: Submit
    my_submitted_samples = api.submit(my_samples)

    # Step 4: Retrieve (I know, they're just up there - just showcasing)
    my_submitted_samples = api.retrieve([sample.accession for sample in my_submitted_samples])

    # Step 5: Update
    my_updated_samples = update_samples(submitted_samples=my_submitted_samples, api=api)

    # Step 6: profit (Aka, just get your spreadsheet and get outta here)
    output_processor = XlsxOutputProcessor('my_updated_samples.xlsx')
    output_processor.save(my_updated_samples)



if __name__ == '__main__':
    webin_username = sys.argv[1]
    webin_password = sys.argv[2]
    main(webin_username, webin_password)


import pytest

from broker.input_processor import TsvInputProcessor
from broker.input_processor import XlsxInputProcessor


@pytest.fixture
def valid_minimal_input():
    return [{'name': 'python_test', 'other_field': 'other_value'}]


@pytest.fixture
def valid_minimal_tsv():
    return TsvInputProcessor(input_data='valid_minimal.tsv')


@pytest.fixture
def valid_minimal_xlsx():
    return XlsxInputProcessor(input_data='valid_minimal.xlsx')


def test_loading_input_tsv(valid_minimal_input, valid_minimal_tsv):
    assert valid_minimal_input == valid_minimal_tsv.input_data


def test_loading_input_xlsx(valid_minimal_input, valid_minimal_xlsx):
    assert valid_minimal_input == valid_minimal_xlsx.input_data



import pytest

from broker.output_processor import TsvOutputProcessor, XlsxOutputProcessor
from broker.metadata_entity import Biosample


@pytest.fixture
def valid_minimal_data():
    return [Biosample({'name': 'python_test', 'other_field': 'other_value'})]


def test_save(valid_minimal_data):



